#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''Run DelogX in shell directly.

**Do not use this in production environment.**

Run `python debug.py` in your terminal, and open `http://127.0.0.1:8000`.
'''
from __future__ import unicode_literals

import os
import sys

from flask import Flask
from DelogX import DelogX

CWD = os.path.dirname(os.path.realpath(__file__))
sys.path.append(CWD)
os.chdir(CWD)

app = Flask(__name__)
delogx = DelogX(CWD, app)
debug_host = delogx.default_conf('debug.host')
debug_port = delogx.default_conf('debug.port')

delogx.framework.run(host=debug_host, port=debug_port)



# -*- coding: utf-8 -*-
'''DelogX.utils.plugin

Base class of any plugin and a plugin manager.
'''
from __future__ import unicode_literals

import errno
import importlib
import os
import re
import sys

from DelogX.entity.config import Config


class Plugin(object):
    '''Base class and interface of any DelogX plugin.

    Attributes:

        app (DelogX): DelogX object.
        manager (PluginManager): PluginManager object of DelogX.
        name (str): Name of this plugin and its directory.
        workspace (str): Path of the directory of this plugin.
    '''
    app = None
    manager = None
    name = 'Plugin'
    workspace = None

    def __init__(self, app):
        '''Iniitialize plugin.

        Args:

            app (DelogX): DelogX object.
        '''
        self.app = app
        self.manager = app.plugin_manager

    def run(self):
        '''Method to call when this plugin is enabled.'''
        pass

    def version(self):
        '''Get the version of plugin itself.

        This is a example to show how to get the meta of plugin itself.

        Returns:

            str: Version of this plugin.
        '''
        info = self.manager.plugins.get(self.name)
        if info and info.get('version'):
            return info.get('version')
        return ''


class PluginManager(object):
    '''Plugin manager of DelogX.

    Attributes:

        directory (str): Absolute path of the plugin directory.
        plugins (dict): Loaded plugins.
        filters (dict): Registered filter hooks.
        actions (dict): Registered action hooks.

    Formats:

        plugins: {
            'plugin_name': {
                'entry': entry,
                'author': author,
                'version': version,
                'description': description
            },
            ...
        }

        filters: {
            'hook_name': [
                (func, priority),
                ...
            ],
            ...
        }

        actions: {
            'hook_name': [
                (func, priority),
                ...
            ],
            ...
        }
    '''
    directory = None
    plugins = None
    filters = None
    actions = None

    def __init__(self, app, directory):
        '''Initialize plugin manager.

        Args:

            app (DelogX): DelogX object.
            directory (str): Name of the plugins directory.
        '''
        self.app = app
        self.plugins = dict()
        self.filters = dict()
        self.actions = dict()
        self.directory = directory
        if not os.path.exists(directory):
            try:
                os.makedirs(directory)
            except OSError as exception:
                if (exception.errno != errno.EEXIST or
                        not os.path.isdir(directory)):
                    raise exception
        init_py = os.path.join(directory, '__init__.py')
        if not os.path.exists(init_py):
            open(init_py, 'a').close()
        sys.path.append(directory)

    def load_all(self):
        '''Load all plugins in the plugin directory.'''
        plugin_list = os.listdir(self.directory)
        for plugin in plugin_list:
            self.load(plugin)

    def enable_all(self):
        '''Enable allloaded plugins.'''
        for plugin in self.plugins:
            self.enable(plugin)

    def load(self, plugin_name):
        '''Load a plugin by name.

        Args:

            plugin_name (str): Name of the plugin.

        Returns:

            object: Entry object of the plugin.
        '''
        plugin_dir = os.path.join(self.directory, plugin_name)
        plugin_meta = os.path.join(plugin_dir, 'plugin.json')
        name_re = re.compile(r'^[A-Za-z_][0-9A-Za-z_]*$')
        if (not os.path.isfile(plugin_meta) or
                name_re.match(plugin_name) is None):
            return None
        init_py = os.path.join(plugin_dir, '__init__.py')
        if not os.path.exists(init_py):
            open(init_py, 'a').close()
        meta = Config(plugin_meta)
        entry_name = meta.get('entry').strip(". \n\r\t")
        author = meta.get('author', '')
        version = meta.get('version', '')
        description = meta.get('description', '')
        if not entry_name or plugin_name in self.plugins:
            return None
        module_name = [plugin_name]
        module_name.extend(entry_name.split('.')[:-1])
        module_name = '.'.join(module_name)
        class_name = ''.join(entry_name.split('.')[-1:])
        module = importlib.import_module(module_name)
        entry = getattr(module, class_name)(self.app)
        entry.name = plugin_name
        entry.workspace = plugin_dir
        self.plugins[plugin_name] = {
            'entry': entry,
            'author': author,
            'version': version,
            'description': description
        }
        return entry

    def enable(self, plugin_name):
        '''Call `run()` method of the plugin entry object.'''
        if plugin_name in self.plugins:
            self.plugins[plugin_name]['entry'].run()

    def add_filter(self, hook_name, func, priority=10):
        '''Register a filter function on a hook.

        Args:

            hook_name (str): Name of the hook registered on.
            func (function): Function needs to register.
            priority (int): Priority of the function on the hook, defaults 10.
        '''
        if not isinstance(self.filters.get(hook_name), list):
            self.filters[hook_name] = list()
        self.filters[hook_name].append((func, priority))
        self.filters[hook_name] = sorted(
            self.filters[hook_name], key=lambda x: x[1])

    def add_action(self, hook_name, func, priority=10):
        '''Register an action function on a hook.

        Args:

            hook_name (str): Name of the hook registered on.
            func (function): Function needs to register.
            priority (int): Priority of the function on the hook, defaults 10.
        '''
        if not isinstance(self.actions.get(hook_name), list):
            self.actions[hook_name] = list()
        self.actions[hook_name].append((func, priority))
        self.actions[hook_name] = sorted(
            self.actions[hook_name], key=lambda x: x[1])

    def do_filter(self, hook_name, item):
        '''Call a filter hook to filter an object.

        Args:

            hook_name (str): Name of the hook to call.
            item (object): Object to be filtered.

        Returns:

            object: Filtered object.
        '''
        if hook_name not in self.filters:
            return item
        for hook in self.filters[hook_name]:
            item = hook[0](item)
        return item

    def do_action(self, hook_name, *args, **kwargs):
        '''Call an action hook to do an action.

        Args:

            hook_name (str): Name of the hook to call.
            *args: Parameters.
            **kwargs: Keyword parameters.
        '''
        if hook_name not in self.actions:
            return
        for hook in self.actions[hook_name]:
            hook[0](*args, **kwargs)



# -*- coding: utf-8 -*-
'''DelogX.utils.parser

Utils of markdown parser.
'''
from __future__ import unicode_literals

import markdown
from markdown.extensions import Extension


class Markdown(object):
    '''Include the markdown parser method.'''

    @classmethod
    def markdown(cls, text, exts=None):
        '''Parse markdown text to HTML5.

        Args:

            text (str): Raw markdown text.
            ext (list of str/class): Extensions of markdown module.

        Returns:

            str: Parsed HTML text.
        '''
        ext_list = [
            'markdown.extensions.attr_list',
            'markdown.extensions.tables',
            'markdown.extensions.fenced_code',
            DelExtension()
        ]
        if exts and isinstance(exts, list):
            ext_list.extend(exts)
        elif exts:
            ext_list.append(exts)
        return markdown.markdown(
            text,
            output_format='html5',
            tab_length=4,
            extensions=ext_list
        )


class DelExtension(Extension):
    '''Extension to provide `<del>` tag support.'''

    def extendMarkdown(self, md, md_globals):
        '''Add `<del>` tag support to markdown.'''
        from markdown.inlinepatterns import SimpleTagPattern
        md_del_re = r'(\~\~)(.+?)(\~\~)'
        md.inlinePatterns.add(
            'del', SimpleTagPattern(md_del_re, 'del'), '<not_strong')



# -*- coding: utf-8 -*-
'''DelogX.entity.item

Define class and interface of a item.

An item is a post or page.
'''
from __future__ import unicode_literals

import codecs
import os
import re

from DelogX.utils.parser import Markdown
from DelogX.utils.path import Path


class Item(object):
    '''Base class of items.

    Attributes:

        filename (str): Name of the item file.
        path (str): Absolute path of the item file.
        hidden (bool): Whether this item is hidden.
        content (str): Parsed content of this item.
        url (str): Clean URL of this item.
        cooked_url (str): Encoded URL of this item.
        title (str): Title of this item.
        title_mode (int): Type of title, 0 is nil, 1 is Atx, 2 is Setext.
    '''
    filename = None
    path = None
    hidden = False
    content = None
    url = None
    title = None
    title_mode = 0

    def __init__(self, app, filename, path):
        '''Initialize and load an item.

        Args:

            app (DelogX): DelogX object.
            filename (str): Name of this item file.
            path (str): Name of items directory.
        '''
        self.app = app
        self.filename = filename
        self.path = os.path.join(path, filename)
        self.update()

    def __str__(self):
        return '{0}({1})'.format(self.__class__.__name__, repr(self.filename))

    def update(self):
        '''Load the meta and content of this item.'''
        self.load_meta()
        self.load_content()

    def valid(self):
        '''Return whether this item exists and is valid.

        Returns:

            bool: Whether the item is valid.
        '''
        return self.filename is not None and os.path.isfile(self.path)

    def load_meta(self):
        '''Load the meta of this item, include `hidden`, `url` and `title`.'''
        if not self.valid():
            return
        self.hidden = self.filename.startswith('.')
        self.url = os.path.splitext(self.filename)[0]
        if self.hidden:
            self.url = self.url[1:]
        atx_re = re.compile(r'^\s*#\s*([^#]+)\s*#*\s*$')
        setext_re = re.compile(r'^\s*=+\s*$')
        with codecs.open(self.path, encoding='utf-8') as item_file:
            line1 = item_file.readline()
            line2 = item_file.readline()
        line1 = line1.strip('\n').strip('\r')
        line2 = line2.strip('\n').strip('\r')
        atx_match = atx_re.match(line1)
        setext_match = setext_re.match(line2)
        if atx_match and atx_match.group(1):
            self.title = atx_match.group(1)
            self.title_mode = 1
        elif setext_match and line1.strip():
            self.title = line1
            self.title_mode = 2
        else:
            self.title = self.url
            self.title_mode = 0

    def load_content(self):
        '''Load and parse the content of this item.'''
        lines = Path.read_file(self.path)
        if lines is None:
            self.content = None
        if self.title_mode == 1:
            lines = lines[1:]
        elif self.title_mode == 2:
            lines = lines[2:]
        self.content = ''.join(lines)
        self.content = Markdown.markdown(self.content, self.app.markdown_ext)


class Post(Item):
    '''Form as a post.

    Attributes:

        time (float): Modification timestamp of this post.
        cooked_time (str): Formatted modification time.
    '''
    time = None

    def load_meta(self):
        '''Load the meta of this post, include timestamp `time`.'''
        super(Post, self).load_meta()
        self.time = os.path.getmtime(self.path)


class Page(Item):
    '''Form as a page.

    Attributes:

        sort (int): Customize sort order of this page.
    '''
    sort = None

    def load_meta(self):
        '''Load the meta of this page, include sort order `sort`.'''
        super(Page, self).load_meta()
        sort_re = re.compile(r'^\.\d+$')
        match = sort_re.match(os.path.splitext(self.url)[1])
        if match:
            self.url = os.path.splitext(self.url)[0]
            if self.title_mode == 0:
                self.title = self.url
            self.sort = int(match.group()[1:])



# -*- coding: utf-8 -*-
'''DelogX.entity.config

Class to load, save and modify json config files.
'''
from __future__ import unicode_literals

import codecs
import errno
import json
import os

from collections import OrderedDict


class Config(object):
    '''Config manager of DelogX.

    Attributes:

        configfile (str): Name of the config file.
        config (object): Config data object.
    '''
    configfile = None
    config = None

    def __init__(self, filename=None):
        '''Initialize config manager.

        Args:

            filename (str, optional): Name of the config file.
        '''
        if filename is None:
            self.config = OrderedDict()
        else:
            self.configfile = filename
            if not self.load():
                self.config = OrderedDict()

    def __str__(self):
        try:
            return json.dumps(self.config, indent=4)
        except TypeError:
            return None

    def load(self, loadfrom=None):
        '''Load data from a json file.

        Args:

            loadfrom (str): If set, load from this file.

        Returns:

            bool: Whether load successful.
        '''
        if loadfrom is None:
            loadfrom = self.configfile
        if loadfrom is None or not os.path.isfile(loadfrom):
            return False
        with codecs.open(loadfrom, encoding='utf-8') as load_file:
            lines = load_file.readlines()
        if not lines:
            return False
        try:
            content = ''.join(lines)
            self.config = json.loads(content, object_pairs_hook=OrderedDict)
            return True
        except ValueError:
            return False

    def save(self, saveto=None):
        '''Save data to a json file.

        Args:

            saveto (str): If set, save to this file.

        Returns:

            bool: Whether save successful.
        '''
        if saveto is None:
            saveto = self.configfile
        if saveto is None:
            return False
        try:
            content = json.dumps(self.config, indent=4)
        except TypeError:
            return False
        saveto_dir = os.path.dirname(saveto)
        if saveto_dir and not os.path.exists(saveto_dir):
            try:
                os.makedirs(saveto_dir)
            except OSError as exception:
                if (exception.errno != errno.EEXIST or
                        not os.path.isdir(saveto_dir)):
                    return False
        with codecs.open(saveto, 'w', encoding='utf-8') as save_file:
            try:
                save_file.write(content)
                return True
            except IOError:
                return False

    def let(self, key, value):
        '''Set the data of a config entry.

        Args:

            key (str): Key of the entry.
            value (object): Value needs to set.

        Returns:

            object: If set successful return the value.
        '''
        return self._let_config(self.config, key, value)

    def get(self, key, default=None):
        '''Get the value of a config entry.

        Args:

            key (str): Key of the entry.
            default (object, optional): Return default value if no such entry.

        Returns:

            object: Value of the entry.
        '''
        value = self._get_config(self.config, key)
        return value if value is not None else default

    def delete(self, key):
        '''Delete the data of a config entry.

        Args:

            key (str): Key of the entry.

        Returns:

            object: If delete successful return the value of the entry.
        '''
        return self._delete_config(self.config, key)

    def _let_config(self, item, key, value):
        if not isinstance(item, OrderedDict):
            return None
        if '.' in key:
            k, rest = key.split('.', 1)
            if k not in item:
                item[k] = OrderedDict()
            return self._let_config(item[k], rest, value)
        else:
            item[key] = value
            return value

    def _get_config(self, item, key):
        if not isinstance(item, OrderedDict):
            return None
        if '.' in key:
            k, rest = key.split('.', 1)
            if k not in item:
                return None
            else:
                return self._get_config(item[k], rest)
        else:
            if key in item:
                return item[key]
            else:
                return None

    def _delete_config(self, item, key):
        if not isinstance(item, OrderedDict):
            return None
        if '.' in key:
            k, rest = key.split('.', 1)
            if k not in item:
                return None
            else:
                return self._delete_config(item[k], rest)
        else:
            if key in item:
                return item.pop(key, None)
            else:
                return None



# -*- coding: utf-8 -*-
'''DelogX.utils.compat

Utils about compatibility of OS and Python.
'''
from __future__ import unicode_literals

import codecs
import os
import platform
import sys


class Compat(object):
    '''Compatibility utils.'''

    @classmethod
    def version(cls):
        '''Return the major version of Python.

        Returns:

            int: Major version of Python.
        '''
        return sys.version_info.major

    @classmethod
    def sys(cls):
        '''Return the name of the operating system.

        Returns:

            str: Name of the operating system.
        '''
        return platform.system()

    @classmethod
    def is_wsl(cls):
        '''Whether DelogX is running on WSL (Windows Subsystem for Linux).'''
        def _read_file(filename):
            if not os.path.isfile(filename):
                return ''
            with codecs.open(filename, encoding='utf-8') as some_file:
                lines = some_file.readlines()
            return ''.join(lines)
        if Compat.sys() != 'Linux':
            return False
        version = _read_file('/proc/version')
        osrelease = _read_file('/proc/sys/kernel/osrelease')
        if 'Microsoft' in version or 'Microsoft' in osrelease:
            return True
        return False

    @classmethod
    def unicode_convert(cls, string, to_byte=True):
        '''Convert a string to unicode or byte.

        Args:

            string (str): String needs to convert.
            to_byte (bool): Whether convert to byte, defaults True.

        Returns:

            str: Converted string.
        '''
        if Compat.version() < 3:
            if to_byte:
                return string.encode('utf-8')
            else:
                return string.decode('utf-8')
        return string

    @classmethod
    def cmp_to_key(cls):
        '''Return a class which convert cmp function to key function.

        Returns:

            class: Class to convert cmp function to key function.
        '''
        return CmpToKey


class CmpToKey(object):
    '''Convert cmp function to key function in `sorted()`.'''

    def __init__(self, obj):
        self.obj = obj

    def __lt__(self, other):
        return self.sort_compare(self.obj, other.obj) < 0

    def __gt__(self, other):
        return self.sort_compare(self.obj, other.obj) > 0

    def __eq__(self, other):
        return self.sort_compare(self.obj, other.obj) == 0

    def __le__(self, other):
        return self.sort_compare(self.obj, other.obj) <= 0

    def __ge__(self, other):
        return self.sort_compare(self.obj, other.obj) >= 0

    def __ne__(self, other):
        return self.sort_compare(self.obj, other.obj) != 0

    @classmethod
    def sort_compare(cls, item_a, item_b):
        '''The cmp function to compare `DelogX.entity.item.Page`.

        Args:

            item_a (object): First object to compare.
            item_b (object): Second object.

        Returns:

            int: 0 is `a = b`, positive is `a > b`, negative is `a < b`.
        '''
        a_sort = item_a[1].sort
        b_sort = item_b[1].sort
        if a_sort is not None and b_sort is not None:
            return a_sort - b_sort
        elif a_sort is None and b_sort is None:
            return 0
        elif a_sort is None:
            return 1
        else:
            return -1



# -*- coding: utf-8 -*-
'''DelogX.entity.module

Class to check whether modules ready.
'''
from __future__ import unicode_literals


class Module(object):
    '''Module checker of DelogX

    Attributes:

        modules_require (list of str): Required modules.
    '''
    modules_require = [
        'flask',
        'watchdog',
        'markdown'
    ]

    @classmethod
    def has(cls, module):
        '''Check whether a module is installed.

        Args:

            module (str): Name of the module.

        Returns:

            bool: Whether the module is installed.
        '''
        try:
            __import__(module)
            return True
        except ImportError:
            return False

    @classmethod
    def ready(cls):
        '''Check whether all required modules are installed.

        Returns:

            bool: Whether all required modules are installed.
        '''
        return Module.missing() is None

    @classmethod
    def missing(cls):
        '''Return names of the missing modules.

        Returns:

            list of str: Names of the missing modules.
        '''
        modules_missing = list()
        for module in Module.modules_require:
            if not Module.has(module):
                modules_missing.append(module)
        if modules_missing:
            return modules_missing
        return None



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import datetime
import os

from flask import request
from werkzeug.contrib.atom import AtomFeed

from DelogX.entity.config import Config
from DelogX.utils.path import Path
from DelogX.utils.plugin import Plugin


class DelogFeed(Plugin):

    config = None
    feed_url = None
    feed_limit = 10

    def run(self):
        self.config = Config(os.path.join(self.workspace, 'config.json'))
        self.feed_url = self.config.get('delog_feed.url')
        self.feed_limit = self.config.get('delog_feed.limit')
        if not self.feed_url:
            self.feed_url = '/feed'
        if not self.feed_limit:
            self.feed_limit = 10
        self.feed_url = Path.format_url(self.feed_url)
        if not self.feed_url.endswith('/'):
            self.feed_url += '/'
        self.app.add_url_rule(self.feed_url, 'delog_feed', self.make_feed)
        self.manager.add_filter('dx_render', self.add_link)

    def add_link(self, render):
        site_name = self.app.default_conf('site.name')
        copy = render.lower()
        search = '<head>'
        tag = (
            '<link href="{href}" rel="alternate" '
            'title="{title}" type="application/atom+xml">').format(
                href=self.feed_url, title=site_name)
        try:
            index = copy.index(search) + len(search)
            render = render[:index] + tag + render[index:]
        except ValueError:
            render = render + tag
        return render

    def make_feed(self):
        site_name = self.app.default_conf('site.name')
        feed = AtomFeed(site_name, feed_url=request.url, url=request.url_root)
        posts = self.app.post_bundle.get_list(1, self.feed_limit)
        for post in posts:
            post = self.manager.do_filter('dx_post', post)
            ext_url = request.url_root
            if ext_url.endswith('/'):
                ext_url = ext_url[:-1]
            ext_url = ext_url + post.cooked_url
            update_time = datetime.datetime.fromtimestamp(post.time)
            feed.add(
                title=post.title, title_type='text',
                content=post.content, content_type='html',
                url=ext_url, author=site_name,
                updated=update_time, published=update_time)
        return feed.get_response()



# -*- coding: utf-8 -*-
'''DelogX.utils.watch

The watchdog handler of DelogX bundles.
'''
from __future__ import unicode_literals

import os

from watchdog.events import PatternMatchingEventHandler


class Watch(PatternMatchingEventHandler):
    '''Event handler to watch file changing event in some directory.

    Attributes:

        bundle (Bundle): Bundle object relates to the watched directory.
    '''
    bundle = None

    def __init__(self, app, bundle, patterns, is_page=False):
        '''Initialize changing event handler.

        Args:

            app (DelogX): DelogX object.
            bundle (Bundle): Bundle object relates to the watched directory.
            patterns (list of str): Patterns to allow matching event paths.
            is_page (bool): Whether the bundle is a PageBundle.
        '''
        super(Watch, self).__init__(
            patterns=patterns, ignore_directories=True)
        self.app = app
        self.bundle = bundle
        self.is_page = is_page

    def on_created(self, event):
        self.do_update(os.path.split(event.src_path)[1])

    def on_deleted(self, event):
        self.do_remove(os.path.split(event.src_path)[1])

    def on_modified(self, event):
        self.do_update(os.path.split(event.src_path)[1])

    def on_moved(self, event):
        self.do_remove(os.path.split(event.src_path)[1])
        self.do_update(os.path.split(event.dest_path)[1])

    def do_update(self, filename):
        '''Update or Add a file to bundle.

        If the bundle is a PageBundle, update the header.

        Args:

            filename (str): Name of the file.
        '''
        self.bundle.update(filename)
        if self.is_page:
            self.app.update_header()

    def do_remove(self, filename):
        '''Remove a file in bundle.

        If the bundle is a PageBundle, update the header.

        Args:

            filename (str): Name of the file.
        '''
        self.bundle.remove(filename)
        if self.is_page:
            self.app.update_header()



# -*- coding: utf-8 -*-
'''The class, definition and interface of DelogX.'''
from __future__ import unicode_literals

import os
import time

from flask import render_template, abort, send_from_directory
from watchdog.observers import Observer
from watchdog.observers.polling import PollingObserver

from DelogX.entity.bundle import PostBundle, PageBundle
from DelogX.entity.config import Config
from DelogX.utils.compat import Compat
from DelogX.utils.i18n import I18n
from DelogX.utils.path import Path
from DelogX.utils.plugin import PluginManager
from DelogX.utils.watch import Watch


class DelogX(object):
    '''The entry and interface of DelogX.

    Attributes:

        framework (Flask): Flask application object.
        config (Config): Config manager of DelogX.
        runtime (Config): Runtime config manager of DelogX.
        header (list of Page): Pages shown in navbar.
        post_bundle (Bundle): Item bundle of DelogX posts.
        page_bundle (Bundle): Item bundle of DelogX pages.
        observer (Observer): Observer of watchdog module.
        i18n (I18n): I18n manager of DelogX.
        markdown_ext (list of str/class): Extensions of markdown module.
        plugin_manager (PluginManager): Plugin manager of DelogX.
    '''

    def __init__(self, app_path, framework, config='config.json'):
        '''Initialize DelogX object.

        Args:

            app_path (str): Absolute path of the blog application.
            framework (Flask): Flask application object.
            config (str): Config file of DelogX, defaults `config.json`.
        '''
        self.framework = framework
        app_path = os.path.realpath(app_path)
        config = Path.abs_path(app_path, config)
        config = Config(config)
        runtime = Config()
        module_path = os.path.dirname(os.path.realpath(__file__))
        runtime.let('path.app', app_path)
        runtime.let('path.module', module_path)
        self.config = config
        self.runtime = runtime
        self.markdown_ext = list()
        self.init_runtime()
        self.init_route()
        self.init_plugins()
        self.update_header()

    def init_runtime(self):
        '''Initialize runtime environments of DelogX.'''
        conf = self.default_conf
        runtime = self.runtime
        app_path = runtime.get('path.app')
        module_path = runtime.get('path.module')
        init_path_list = [
            'directory.post',
            'directory.page',
            'directory.static',
            'directory.themes',
            'directory.plugins'
        ]
        init_url_list = [
            'url_prefix.post',
            'url_prefix.page',
            'url_prefix.static',
            'url_prefix.post_list'
        ]
        for key in init_path_list:
            runtime.let(key, Path.abs_path(app_path, conf(key)))
        for key in init_url_list:
            runtime.let(key, Path.format_url(conf(key)))
        post_dir = runtime.get('directory.post')
        page_dir = runtime.get('directory.page')
        themes_dir = runtime.get('directory.themes')
        theme = conf('local.theme')
        theme = 'default' if not theme else theme
        theme_path = os.path.join(themes_dir, theme)
        self.framework.template_folder = theme_path
        self.post_bundle = PostBundle(
            self, app_path, post_dir, conf('local.list_size'))
        self.page_bundle = PageBundle(self, app_path, page_dir)
        post_watch = Watch(self, self.post_bundle, ['*.md'])
        page_watch = Watch(self, self.page_bundle, ['*.md'], is_page=True)
        watch_polling = conf('local.watch_polling')
        watch_polling = True if Compat.is_wsl() else watch_polling
        self.observer = PollingObserver() if watch_polling else Observer()
        self.observer.setDaemon(True)
        self.observer.schedule(post_watch, post_dir)
        self.observer.schedule(page_watch, page_dir)
        self.observer.start()
        self.i18n = I18n(
            Path.format_url(module_path, 'locale'), conf('local.locale'))

    def init_route(self):
        '''Initialize URL routes of DelogX.'''
        runtime = self.runtime
        static_rule = Path.format_url(
            runtime.get('url_prefix.static'), '<static_file>')
        list_rule = Path.format_url(
            runtime.get('url_prefix.post_list'), '<int:number>/')
        item_rule = Path.format_url(
            runtime.get('url_prefix.page'), '<item_id>/')
        page_rule = Path.format_url(
            runtime.get('url_prefix.page'), '<page_id>/')
        post_rule = Path.format_url(
            runtime.get('url_prefix.post'), '<post_id>/')
        icon_rule = Path.format_url(
            runtime.get('url_prefix.static'), 'favicon.ico')
        self.add_url_rule('/', 'delogx_index', self.route_index)
        self.add_url_rule(static_rule, 'delogx_static', self.route_static)
        self.add_url_rule(list_rule, 'delogx_list', self.route_list)
        if runtime.get('url_prefix.post') == runtime.get('url_prefix.page'):
            self.add_url_rule(item_rule, 'delogx_page', self.route_item)
        else:
            self.add_url_rule(page_rule, 'delogx_page', self.route_page)
            self.add_url_rule(post_rule, 'delogx_post', self.route_post)
        self.framework.add_url_rule(
            '/favicon.ico', 'delogx_favicon', redirect_to=icon_rule)
        self.framework.errorhandler(404)(self.route_not_found)

    def init_plugins(self):
        '''Initialize plugin manager of DelogX.'''
        plugins_dir = self.runtime.get('directory.plugins')
        self.plugin_manager = PluginManager(self, plugins_dir)
        manager = self.plugin_manager
        manager.add_filter('dx_page', self.cook_page, 0)
        manager.add_filter('dx_post', self.cook_post, 0)
        manager.add_filter('dx_static', self.cook_static, 0)
        manager.load_all()
        manager.enable_all()

    def default_conf(self, key):
        '''Get a config with default value.

        Args:

            key (str): Key of the config entry.

        Returns:

            object: Value of the config entry.
        '''
        conf = self.config.get
        default_dict = {
            'site.name': 'DelogX',
            'site.subname': 'Yet another Markdown based blog',
            'local.theme': 'default',
            'local.locale': 'en_US',
            'local.list_size': 10,
            'local.watch_polling': False,
            'local.time_format': '%Y-%m-%d %H:%M',
            'url_prefix.post': '/post',
            'url_prefix.page': '/page',
            'url_prefix.post_list': '/list',
            'url_prefix.static': '/static',
            'directory.post': 'posts',
            'directory.page': 'pages',
            'directory.static': 'static',
            'directory.themes': 'themes',
            'directory.plugins': 'plugins',
            'static.css': [],
            'static.js': [],
            'debug.host': '0.0.0.0',
            'debug.port': 8000
        }
        return default_dict.get(key) if conf(key) is None else conf(key)

    def add_url_rule(self, rule, endpoint, func):
        '''Add an URL route rule to Flask object.

        Args:

            rule (str): URL rule string.
            endpoint (str): Endpoint for the registered URL rule.
            func (function): Function to call when request.
        '''
        self.framework.add_url_rule(rule, endpoint, func)

    def update_header(self):
        '''Reload pages shown in navbar.

        Hidden pages will be ignored.
        '''
        header = self.page_bundle.get_list()
        if header is None:
            header = list()
        for page in header:
            page = self.plugin_manager.do_filter('dx_page', page)
        self.plugin_manager.do_action('dx_header')
        self.header = self.plugin_manager.do_filter('dx_header', header)

    def get_render(self, template, **context):
        '''Return rendered HTML content.

        Args:

            template (str): Filename of the template.
            **context: Context of the template.

        Returns:

            str: rendered HTML.

        Contexts:

            app: DelogX itself.
            _g: Method to get i18n text.
            _c: Method to get config.
            _rt: Method to get runtime config.
            _css: CSS list of site.
            _js: JS list of site.
        '''
        conf = self.default_conf
        render = render_template(
            template,
            app=self,
            _g=self.i18n.get,
            _c=self.default_conf,
            _rt=self.runtime.get,
            _css=self.get_static(conf('static.css')),
            _js=self.get_static(conf('static.js')),
            **context)
        self.plugin_manager.do_action('dx_render')
        return self.plugin_manager.do_filter('dx_render', render)

    def get_page(self, page_id):
        '''Return cooked Page object by id.

        Args:

            page_id (str): Request ID of the page.

        Returns:

            Page: Cooked Page object.
        '''
        page_id = Path.urldecode(page_id)
        page = self.page_bundle.get(page_id)
        self.plugin_manager.do_action('dx_page')
        page = self.plugin_manager.do_filter('dx_page', page)
        return page

    def cook_page(self, page):
        '''Cook a Page.

        Add prefix to the URL of the Page.

        Args:

            page (Page): Page needs to cook.

        Returns:

            Page: Cooked Page.
        '''
        runtime = self.runtime.get
        if not page:
            return None
        page_url = runtime('url_prefix.page')
        page.cooked_url = Path.format_url(page_url, Path.urlencode(page.url))
        return page

    def get_post(self, post_id):
        '''Return cooked Post object by id.

        Args:

            post_id (str): Request ID of the post.

        Returns:

            Post: Cooked Post object.
        '''
        post_id = Path.urldecode(post_id)
        post = self.post_bundle.get(post_id)
        self.plugin_manager.do_action('dx_post')
        return self.plugin_manager.do_filter('dx_post', post)

    def cook_post(self, post):
        '''Cook a Post.

        Add prefix to the URL of the Post and convert timestamp to string.

        Args:

            post (Post): Post needs to cook.

        Returns:

            Post: Cooked Post.
        '''
        conf = self.config.get
        runtime = self.runtime.get
        if not post:
            return None
        post_url = runtime('url_prefix.post')
        time_format = conf('local.time_format')
        post.cooked_url = Path.format_url(post_url, Path.urlencode(post.url))
        post.cooked_time = time.strftime(
            Compat.unicode_convert(time_format),
            time.localtime(post.time))
        post.cooked_time = Compat.unicode_convert(
            post.cooked_time,
            to_byte=False)
        return post

    def get_static(self, statics):
        '''Return a list of static files.

        Args:

            statics (list of str): Static files.

        Returns:

            list: Cooked static files.
        '''
        self.plugin_manager.do_action('dx_static')
        return self.plugin_manager.do_filter('dx_static', statics)

    def cook_static(self, statics):
        '''Cook a list of static files.

        Convert all relative links to absolute.

        Args:

            statics (list of str): Static files.

        Returns:

            list: Cooked list of static files.
        '''
        runtime = self.runtime.get
        static_url = runtime('url_prefix.static')
        if not statics:
            statics = list()
        for i, link in enumerate(statics):
            if not link.startswith(('/', 'http://', 'https://')):
                statics[i] = Path.format_url(static_url, link)
        return statics

    def route_static(self, static_file):
        '''Response a static file.

        Search the file in static directory firstly, then search in
        theme directory.

        Args:

            static_file (str): Relative path of the static file.

        Returns:

            object: Static file.

        Abort:

            404: No such static file.
        '''
        static_dir = self.runtime.get('directory.static')
        themes_dir = self.framework.template_folder
        static_path = os.path.join(static_dir, static_file)
        themes_path = os.path.join(themes_dir, 'static', static_file)
        if os.path.isfile(static_path):
            return send_from_directory(
                os.path.dirname(static_path),
                os.path.basename(static_path))
        elif os.path.isfile(themes_path):
            return send_from_directory(
                os.path.dirname(themes_path),
                os.path.basename(themes_path))
        abort(404)

    def route_index(self):
        '''Response the home page.

        Home page, i.e. the first page of posts.

        Returns:

            str: Rendered HTML of the home page.
        '''
        return self.route_list(1)

    def route_list(self, number):
        '''Response a list of posts.

        Hidden posts will be ignored.

        Args:

            number (int): Page number of the list.

        Returns:

            str: Rendered HTML of the posts list.

        Abort:

            404: No such page of posts list.
        '''
        runtime = self.runtime.get
        post_count = self.post_bundle.get_list_count()
        post_list = self.post_bundle.get_list(number)
        if post_list is None:
            abort(404)
        for post in post_list:
            post = self.plugin_manager.do_filter('dx_post', post)
        prev_page = next_page = True
        if number == 1:
            prev_page = False
        if number == post_count:
            next_page = False
        list_url = Path.format_url(runtime('url_prefix.post_list'))
        url = list_url if list_url.endswith('/') else list_url + '/'
        return self.get_render(
            'list.html', posts=post_list,
            list_id=number, list_url=url,
            prev_page=prev_page, next_page=next_page)

    def route_page(self, page_id):
        '''Response a page.

        Args:

            page_id (str): Request ID of the page.

        Returns:

            str: Rendered HTML of the page.

        Abort:

            404: No such page.
        '''
        page = self.get_page(page_id)
        if page:
            return self.get_render('page.html', page=page)
        abort(404)

    def route_post(self, post_id):
        '''Response a post.

        Args:

            post_id (str): Request ID of the post.

        Returns:

            str: Rendered HTML of the post.

        Abort:

            404: No such post.
        '''
        post = self.get_post(post_id)
        if post:
            return self.get_render('post.html', post=post)
        abort(404)

    def route_item(self, item_id):
        '''Response a page or post.

        Args:

            item_id (str): Request ID of the item.

        Returns:

            str: Rendered HTML of item.

        Abort:

            404: No such item.
        '''
        page = self.get_page(item_id)
        post = self.get_post(item_id)
        if page:
            return self.get_render('page.html', page=page)
        elif post:
            return self.get_render('post.html', post=post)
        abort(404)

    def route_not_found(self, error):
        '''Response `404 Not Found` page.

        Args:

            error (object): Error code of the request.

        Returns:

            str: Rendered HTML of 404.
        '''
        return self.get_render('404.html'), error.code



# -*- coding: utf-8 -*-
'''The command line manager of DelogX.'''
from __future__ import print_function, unicode_literals

import argparse
import os
import shutil
import sys

VERSION = '1.0.7'


def copytree(src, dst):
    '''Copy all files in src to dst.'''
    basename = os.path.basename(src)
    if basename == '__pycache__':
        return
    if not os.path.exists(dst):
        os.makedirs(dst)
    for item in os.listdir(src):
        sfile = os.path.join(src, item)
        dfile = os.path.join(dst, item)
        ext = os.path.splitext(os.path.basename(sfile))[1]
        if os.path.isdir(sfile):
            copytree(sfile, dfile)
        elif ext != '.pyc':
            shutil.copyfile(sfile, dfile)


def init(init_args):
    '''Create a new blog application.'''
    module_path = os.path.dirname(os.path.realpath(__file__))
    sys.path.append(os.path.dirname(module_path))
    from DelogX.entity.config import Config
    from DelogX.utils.compat import Compat
    cwd = os.getcwd()
    defaults = os.path.join(module_path, 'defaults')
    if os.listdir(cwd) and not init_args.force:
        print(
            '''The current directory is not empty,\n'''
            '''use `delogx init --force' if you want to overwrite.'''
        )
        return
    locale_dir = os.path.join(module_path, 'locale')
    locales = list()
    for lang in os.listdir(locale_dir):
        if os.path.isdir(os.path.join(locale_dir, lang)):
            locales.append(lang)
    locale = ''
    while locale not in locales:
        print('Choose Language({0}):\n> '.format(','.join(locales)), end='')
        if Compat.version() == 2:
            locale = raw_input()
        else:
            locale = input()
    copytree(defaults, cwd)
    config = Config(os.path.join(cwd, 'config.json'))
    config.let('local.locale', locale)
    config.save()
    post_dir = os.path.join(cwd, 'posts')
    page_dir = os.path.join(cwd, 'pages')
    if os.path.isfile(post_dir):
        os.remove(post_dir)
    if os.path.isfile(page_dir):
        os.remove(page_dir)
    if not os.path.exists(post_dir):
        os.makedirs(post_dir)
    if not os.path.exists(page_dir):
        os.makedirs(page_dir)
    shutil.copyfile(
        os.path.join(locale_dir, locale, 'hello-delogx.md'),
        os.path.join(post_dir, 'hello-delogx.md'))
    shutil.copyfile(
        os.path.join(locale_dir, locale, 'demo.md'),
        os.path.join(page_dir, 'demo.md'))


def main():
    '''Main function of the manager.'''
    parser = argparse.ArgumentParser(
        description='Manage a DelogX blog application.')
    parser.add_argument(
        '-ver', '--version', action='version',
        version='DelogX v{0}'.format(VERSION))
    subparsers = parser.add_subparsers(
        title='sub-commands', description='valid sub-commands',
        metavar='sub-command', help='description')
    init_parser = subparsers.add_parser(
        'init', help='create DelogX blog application at current directory')
    init_parser.add_argument(
        '-f', '--force', help='force creation of the blog',
        action='store_true')
    init_parser.set_defaults(func=init)
    args = parser.parse_args()
    if not vars(args):
        parser.print_help()
        parser.exit(0)
    args.func(args)


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import re

from DelogX.utils.plugin import Plugin


class ExternalLink(Plugin):

    def run(self):
        self.manager.add_filter('dx_page', self.load_link)

    @classmethod
    def load_link(cls, page):
        if not page:
            return None
        link_re = re.compile(
            r'^(?:\<p\>)?\<a[^>]*\shref="([^"]*)"[^>]*\>([^<>]*)'
            r'\<\/a\>(?:\<\/p\>)?$',
            re.IGNORECASE)
        target_re = re.compile(r'target="([^"]*)"', re.IGNORECASE)
        content = page.content.replace('\n', ' ').strip()
        link_match = link_re.match(content)
        if link_match:
            page.link = link_match.group(1).strip()
            link_title = link_match.group(2).strip()
            page.title = link_title if link_title else page.title
            target_search = target_re.search(content)
            page.target = ''
            if target_search:
                page.target = target_search.group(1).replace('\n', '').strip()
            return None
        return page



# -*- coding: utf-8 -*-
'''DelogX.entity.bundle

Define class and interface of a bundle.

A bundle is a set of items.
'''
from __future__ import unicode_literals

import errno
import os
from collections import OrderedDict

from DelogX.entity.item import Post, Page
from DelogX.utils.compat import Compat
from DelogX.utils.path import Path


class Bundle(object):
    '''Base class of bundles.

    Attributes:

        bundle_list (list of Item): List of items in a bundle.
    '''
    bundle_list = None

    def __str__(self):
        return '{0}({1})'.format(
            self.__class__.__name__, repr(list(self.bundle_list.keys())))

    def sort(self):
        '''Sort the list of the bundle.'''
        pass

    def get(self, url):
        '''Return an item by its name(url).

        Args:

            url (str): Name or URL of the item.
        '''
        pass

    def get_list(self):
        '''Return a list of multiple items.'''
        pass


class DelogXBundle(Bundle):
    '''Base class of DelogX bundles.

    Attributes:

        directory (str): Absolute path of the bundle directory.
    '''
    directory = None

    def __init__(self, app, app_path, directory):
        '''Initialize a DelogX bundle.

        Args:

            app (DelogX): DelogX object.
            app_path (str): Absolute path of the blog application.
            directory (str): Name of items directory.
        '''
        self.app = app
        self.bundle_list = OrderedDict()
        self.directory = Path.abs_path(app_path, directory)
        if not os.path.exists(self.directory):
            try:
                os.makedirs(self.directory)
            except OSError as exception:
                if (exception.errno != errno.EEXIST or
                        not os.path.isdir(self.directory)):
                    raise exception
        for filename in os.listdir(self.directory):
            real_path = os.path.join(self.directory, filename)
            if (os.path.isfile(real_path) and
                    os.path.splitext(filename)[1] == '.md'):
                self.update(filename)
        self.sort()

    def update(self, filename):
        '''Update an item in DelogX bundle.

        Args:

            filename (str): Name of the file needs to be updated.
        '''
        pass

    def remove(self, filename):
        '''Remove an item in DelogX bundle.

        Args:

            filename (str): Name of the file needs to be removed.

        Returns:

            bool: Whether the file is removed successfully.
        '''
        if filename in self.bundle_list:
            self.bundle_list.pop(filename)
            return True
        return False

    def sort(self):
        '''Sort the list of the DelogX bundle.'''
        pass

    def get(self, url):
        '''Return an item by its name(url).

        If cannot find item, try to return a hidden item have the same name.

        Args:

            url (str): Name or URL of the item.

        Returns:

            Item: Item found.
        '''
        finder = list()
        finder_hidden = list()
        for item in self.bundle_list.values():
            if item.url != url:
                continue
            if item.hidden:
                finder_hidden.append(item)
            else:
                finder.append(item)
        if finder:
            return finder[0]
        elif finder_hidden:
            return finder_hidden[0]
        else:
            return None

    def get_list(self):
        '''Return a list of multiple items.

        Returns:

            list of Item: List of these items.
        '''
        return list(self.bundle_list.values())


class PostBundle(DelogXBundle):
    '''A set of posts.

    Attributes:

        list_size (int): Count of posts per page .
    '''
    list_size = 10

    def __init__(self, app, app_path, post_dir, list_size=10):
        '''Initialize a post bundle.

        Args:

            app (DelogX): DelogX object.
            app_path (str): Absolute path of the blog application.
            post_dir (str): Name of posts directory.
            list_size (int): Count of posts per page, defaults 10.
        '''
        self.list_size = list_size
        super(PostBundle, self).__init__(app, app_path, post_dir)

    def update(self, filename):
        '''Update an post in bundle.

        Args:

            filename (str): Name of the file needs to be updated.

        Returns:

            bool: Whether the file is updated successfully.
        '''
        post = Post(self.app, filename, self.directory)
        if post.valid():
            self.bundle_list[filename] = post
            self.sort()
            return True
        return False

    def sort(self):
        '''Sort the list of the post bundle by modification time reversed.'''
        self.bundle_list = OrderedDict(sorted(
            self.bundle_list.items(), key=lambda x: x[1].time, reverse=True))

    def get_list_count(self, post_count=None, list_size=None):
        '''Return the count of posts list.

        Args:

            post_count (int): If set, use it as customize count of posts.
            list_size (int): If set, use it as customize size of list.

        Returns:

            int: Count of posts list.
        '''
        if list_size is None:
            list_size = self.list_size
        import math
        if post_count is None:
            post_count = len([
                post for post in self.bundle_list.values() if not post.hidden
            ])
        return int(math.ceil(float(post_count) / float(list_size)))

    def get_list(self, number=0, list_size=None):
        '''Return a list of multiple posts.

        If param number is 0, return the entire bundle.

        Args:

            number (int): Page number of the list, defaults 0.
            list_size (int): If set, use it as customize size of list.

        Returns:

            list of Post: List of these posts.
        '''
        if list_size is None:
            list_size = self.list_size
        if number:
            post_list = [
                post for post in self.bundle_list.values() if not post.hidden
            ]
            list_count = self.get_list_count(len(post_list), list_size)
            if not 0 < number <= list_count:
                return None
            if post_list and list_count > 0:
                post_list = post_list[
                    list_size * (number - 1):list_size * number
                ]
                return post_list
            else:
                return list()
        else:
            return super(PostBundle, self).get_list()


class PageBundle(DelogXBundle):
    '''A set of pages.'''

    def update(self, filename):
        '''Update an page in bundle.

        Args:

            filename (str): Name of the file needs to be updated.

        Returns:

            bool: Whether the file is updated successfully.
        '''
        page = Page(self.app, filename, self.directory)
        if page.valid():
            self.bundle_list[filename] = page
            self.sort()
            return True
        return False

    def sort(self):
        '''Sort the list of the page bundle by customize order.'''
        self.bundle_list = OrderedDict(sorted(
            self.bundle_list.items(), key=Compat.cmp_to_key()))



# -*- coding: utf-8 -*-
'''DelogX.utils.i18n

Utils about i18n, an i18n manager.
'''
from __future__ import unicode_literals

import os
from string import Formatter

from DelogX.entity.config import Config
from DelogX.utils.compat import Compat


class I18nFormatter(Formatter):
    '''Formatter which added default value support.'''

    def __init__(self, default=''):
        self.default = default

    def get_value(self, key, args, kwargs):
        default = self.default
        if Compat.version() == 2:
            if isinstance(key, int) or isinstance(key, long):
                return args[key] if -len(args) <= key < len(args) else default
            elif isinstance(key, str) or isinstance(key, unicode):
                return kwargs.get(key, self.default)
            else:
                super(I18nFormatter, self).get_value(key, args, kwargs)
        else:
            if isinstance(key, int):
                return args[key] if -len(args) <= key < len(args) else default
            elif isinstance(key, str):
                return kwargs.get(key, self.default)
            else:
                super(I18nFormatter, self).get_value(key, args, kwargs)


class I18n(object):
    '''I18n manager of DelogX.

    Attributes:

        directory (str): Name of the i18n directory.
        locale (str): Current locale.
        data (Config): Message data of the current locale.
    '''
    directory = None
    locale = None
    data = None

    def __init__(self, directory, locale):
        '''Initialize i18n manager.

        Args:

            directory (str): Name of the i18n directory.
            locale (str): Current locale.
        '''
        self.directory = directory
        self.locale = locale
        self.data = Config(os.path.join(directory, locale, 'locale.json'))

    def get(self, key, *args):
        '''Return a message by a message key.

        If no such message, return the key.

        Args:

            key (str): Message key.
            *args: Format substitutions.

        Returns:

            str: Formatted message.
        '''
        fmt = I18nFormatter()
        data = self.data
        local_key = data.get(key) if data.get(key) is not None else key
        try:
            return fmt.format(local_key, *args)
        except (KeyError, IndexError, ValueError):
            return local_key



# -*- coding: utf-8 -*-
'''Setup script of DelogX.'''
import os

from setuptools import setup, find_packages

VERSION = '1.0.7'


def find_package_data():
    '''Get data files of DelogX.'''
    cwd = os.getcwd()
    path = os.path.dirname(os.path.realpath(__file__))
    path = os.path.join(path, 'DelogX')
    data_list = list()
    os.chdir(path)
    data_list.append('DESCRIPTION.rst')
    for root, dirs, files in os.walk('defaults'):
        for filename in files:
            data_list.append(os.path.join(root, filename))
    for root, dirs, files in os.walk('locale'):
        for filename in files:
            data_list.append(os.path.join(root, filename))
    os.chdir(cwd)
    return data_list


desc = open(os.path.join('DelogX', 'DESCRIPTION.rst'))
DESCRIPTION = desc.read()
desc.close()
setup(
    name='DelogX',
    version=VERSION,
    packages=find_packages(),
    install_requires=[
        'Flask>=0.11',
        'watchdog',
        'markdown'
    ],
    package_data={
        'DelogX': find_package_data()
    },
    entry_points={
        'console_scripts': [
            'delogx = DelogX.__main__:main'
        ]
    },
    author='deluxghost',
    author_email='deluxghost@gmail.com',
    description='Yet another Markdown based blog',
    long_description=DESCRIPTION,
    license='Apache 2',
    keywords='flask markdown blog framework',
    url='http://delogx.deluxghost.me/'
)



# -*- coding: utf-8 -*-
'''DelogX.utils.path

Utils and functions about path.
'''
from __future__ import unicode_literals

import codecs
import os

from DelogX.utils.compat import Compat


class Path(object):
    '''Some useful path and URL utils and functions.'''

    @classmethod
    def format_url(cls, *args):
        '''Join strings to an URL and ensure the URL starts with a slash.

        Args:

            *args: strings need to be joined.

        Returns:

            str: Formatted URL.
        '''
        if Compat.sys() == 'Windows':
            return os.path.join('/', *args).replace('\\', '/')
        else:
            return os.path.join('/', *args)

    @classmethod
    def urlencode(cls, url):
        '''Return an encoded URL.

        Args:

            url (str): URL needs to be encoded.

        Returns:

            str: Encoded URL.
        '''
        if Compat.version() < 3:
            import urllib
        else:
            import urllib.parse as urllib
        return urllib.quote(Compat.unicode_convert(url))

    @classmethod
    def urldecode(cls, url):
        '''Return an decoded URL.

        Args:

            url (str): URL needs to be decoded.

        Returns:

            str: Decoded URL.
        '''
        if Compat.version() < 3:
            import urllib
        else:
            import urllib.parse as urllib
        return urllib.unquote(Compat.unicode_convert(url))

    @classmethod
    def abs_path(cls, prefix, path):
        '''If the path is not an absolute path, convert it to absolute.

        Args:

            prefix (str): Directory to be added if the path is not absolute.
            path (str): Path needs to check.

        Returns:

            str: Converted absolute path.
        '''
        if not os.path.isabs(path):
            path = os.path.join(prefix, path)
        return os.path.abspath(path)

    @classmethod
    def read_file(cls, filename):
        '''Read the content of a file (UTF-8).

        Args:

            filename (str): Name of the file.

        Returns:

            list of str: Content of the file.
        '''
        if not os.path.isfile(filename):
            return None
        with codecs.open(filename, encoding='utf-8') as some_file:
            lines = some_file.readlines()
        return lines



# -*- coding: utf-8 -*-
'''DelogX.entity.error

Define error interface of DelogX.
'''
from __future__ import unicode_literals


class Error(Exception):
    '''Base class to define the errors of DelogX.'''

    def __init__(self, value):
        super(Error, self).__init__()
        self.value = value

    def __str__(self):
        return self.value
